span.Alert {
 color: #0000ff;
}
span.BaseN {
 color: #007f00;
}
span.BString {
 color: #c9a7ff;
}
span.Char {
 color: #ff00ff;
}
span.Comment {
 color: #cc9900;
}
span.DataType {
 color: #00ff55;
}
span.DecVal {
 color: #00ffff;
}
span.Error {
 color: #ff0000;
}
span.Float {
 color: #5599ff;
}
span.Function {
 color: #3344ff;
}
span.IString {
 color: #ff0000;
}
span.Keyword {
 color: #11ffff;
 font-weight: bold;
}
span.Operator {
 color: #00ff33;
 font-weight: bold;
}
span.Others {
 color: #b03060;
}
span.RegionMarker {
 color: #96b9ff;
}
span.Reserved {
 color: #9b30ff;
 font-weight: bold;
}
span.String {
 color: #ffaa55;
}
span.Variable {
 color: #ffff00;
}
span.Warning {
 color: #0000ff;
}
